@@ -16,6 +16,7 @@ except:
 LOG_LEVEL_DEBUG = logging.DEBUG
 log = logging.getLogger('aptk')
 #log.setLevel(logging.ERROR)
+#log.setLevel(logging.DEBUG)
 
 PatternType = type(re.compile(''))
 
@@ -78,21 +79,27 @@ class Rule:
             try:
                 if log_level <= LOG_LEVEL_DEBUG:
                     parent = ctxget('*RULE*', '')
-                    data = self.data
-                    if isinstance(data, PatternType): data = data.pattern
+#                    data = self.data
+#                    if isinstance(data, PatternType): data = data.pattern
 
                     p = start
                     self.log_debug("%30.30s %20.20s", self, repr(s[p:p+15]), indicator = '{{{ %02d'%i)
 
                 i,r = iterator.next()
 
+            except:
+                log.debug("got exc", exc_info=1) 
+                raise
+
             finally:
                 if log_level <= LOG_LEVEL_DEBUG:
-                    data = self.data
-                    if isinstance(data, PatternType): data = data.pattern
+#                    data = self.data
+#                    if isinstance(data, PatternType): data = data.pattern
                     self.log_debug("%30.30s => %s", self, repr(r), 
                         indicator = '}}} %02d'%i)
 
+            if r is Undef: raise StopIteration
+
             yield r
 
         
@@ -121,16 +128,18 @@ class Rule:
         else: data = repr(data)
         return self.__class__.__name__ + "(" + data + ")"
 
-    def setup(self, G):
+    def setup(self, G, called):
         if self.is_setting_up: return
+
         try:
             self.is_setting_up = True
 
-            self.rule_setup(G)
+            self.rule_setup(G, called)
         finally:
             self.is_setting_up = False
 
-    def rule_setup(self, G):
+
+    def rule_setup(self, G, calling):
         pass
 
     def possible_matches(self, P, s, start=0, end=None):
@@ -169,7 +178,7 @@ class Terminal(Rule):
     recursive = None
 
     def __init__(self):
-        Rule.__init__(self, backtrack=True)
+        Rule.__init__(self, backtrack=False)
 
     def possible_matches(self, P, s, start=0, end=None):
         r = self.match(P, s, start, end)
@@ -245,25 +254,23 @@ class Sequence(Rule):
                 
         def create_match(k, pos, result, end):
             def match_ctx(mob):
-                if isinstance(mob, Lexem):
+                  if isinstance(mob, Lexem):
                     my_result = [mob]
-                else:
+
+                  else:
                     my_result = mob.lexems
 
-                if isinstance(mob, Lexem):
+                  if isinstance(mob, Lexem):
                     ctx('='+mob.name, str(mob))
 
-                for L in mob:
+                  for L in mob:
                     ctx('='+L.name, str(L))
 
-                match = create_match(k+1, mob.end, result + my_result, end)
+                  match = create_match(k+1, mob.end, result + my_result, end)
 
-                for m in match:
+                  for m in match:
                     yield m
 
-                #except StopIteration:
-                #    raise NotMatched
-                
             if k >= seqlen:
                 yield MatchObject(end=pos, lexems=result, start=start_pos, string=s)
 
@@ -300,7 +307,7 @@ class Sequence(Rule):
                     matches = seq[k](P, s, pos, rec_end)
 
                     for mob in matches:
-                        self.log_debug("mob: %s" % mob)
+                        self.log_debug("mob: %s", mob)
 
                         for r in match_ctx(mob):
                             yield r
@@ -311,7 +318,7 @@ class Sequence(Rule):
                     self.log_debug("seq[%s]: %s", k, seq[k])
                     matches = seq[k](P, s, pos, end)
                     for mob in matches:
-                        self.log_debug("mob: %s" % mob)
+                        self.log_debug("mob: %s", mob)
 
                         for r in match_ctx(mob):
                             yield r
@@ -328,12 +335,13 @@ class Sequence(Rule):
             yield result
             if not self.backtrack: break
 
-    def rule_setup(self, G):
+    def rule_setup(self, G, calling):
         called = []
         is_terminal  = True
         for r in self.data:
-            r.setup(G)
-            called.extend(r.called)
+            r.setup(G, calling)
+            for c in r.called:
+                if c not in called: called.append(c)
             is_terminal = is_terminal and r.is_terminal
         self.called = called
 
@@ -443,6 +451,7 @@ class Quantified(Rule):
         while True:
             # here we always are only interested in the first match
             matched = False
+
             for mob in rule(P, s, start, end):
                 i += 1
                 if isinstance(mob, Lexem):
@@ -452,6 +461,7 @@ class Quantified(Rule):
 
                 matched = True
                 start = mob.end
+
                 break
 
             if max is not None:
@@ -459,17 +469,22 @@ class Quantified(Rule):
 
             if not matched: break
 
+        self.log_debug("result (%s): %s", i, result)
         if i >= min:
+            self.log_debug("possible_matches (%s)", i)
             yield MatchObject(end=start, lexems=result, start=start_pos, string=s)
+        else:
+            self.log_debug("no possible_matches (%s)", i)
 
-    def rule_setup(self, G):
+    def rule_setup(self, G, calling):
         called = []
         is_terminal  = True
 
         r = self.data[0]
-        r.setup(G)
+        r.setup(G, calling)
 
-        called.extend(r.called)
+        for c in r.called:
+            if c not in called: called.append(c)
         self.is_terminal = r.is_terminal
         self.called = called
         
@@ -509,15 +524,23 @@ class Quantified(Rule):
 
 class Calling(Rule):
 
-    def rule_setup(self, G):
+    def rule_setup(self, G, calling):
         name = self.data[0]
+
+        if name in calling:
+            self.recursive = name
+
+            return
+
         called = [ name ]
 
         r = getattr(G, name)
 
         if isinstance(r, Rule):
-            r.setup(G)
-            called.extend(r.called)
+            r.setup(G, calling + called)
+            for c in r.called:
+                if c not in called: called.append(c)
+
             self.is_terminal = r.is_terminal
         else:
             # custom rule, what to do?
@@ -580,9 +603,10 @@ class Capturing(Calling):
             self.log_debug("capture: %s, %s", name, mob)
 
             if isinstance(mob, Lexem):
-                mob =  MatchObject(mob=mob, lexems=[mob])
+                mob = MatchObject(mob=mob, lexems=[mob])
 
             yield P.lex(mob, name)
+
             if not self.backtrack: break
         self.log_debug("done capture: %s", name)
 
@@ -655,7 +679,7 @@ class Alternative(Rule):
 
             for mob in matches:
                 matched = True
-                self.log_debug("got %s" % mob)
+                self.log_debug("got %s", mob)
                 yield mob
                 if not self.backtrack: break
                 if isinstance(alt, Terminal): break
@@ -663,13 +687,17 @@ class Alternative(Rule):
             #self.log_debug("done %s: %s" % (i, alt))
             if matched and not self.backtrack: break
 
-    def rule_setup(self, G):
+    def rule_setup(self, G, calling):
         called = []
+
         is_terminal  = True
         for r in self.data:
-            r.setup(G)
-            called.extend(r.called)
+
+            r.setup(G, calling)
+            for c in r.called:
+                if c not in called: called.append(c)
             is_terminal = is_terminal and r.is_terminal
+
         self.called = called
                 
 
